/*
 * html.h
 *
 * Copyright (c) 1995, John D. Kilburg (john@cs.unlv.edu)
 *
 * See copyright.h for more information.
 *
 */
#include "copyright.h"

#ifndef __HTML_H__
#define __HTML_H__ 1

#define ALIGN_LEFT 0
#define ALIGN_CENTER 1
#define ALIGN_RIGHT 2
#define ALIGN_JUSTIFY 3
#define ALIGN_TOP 4
#define ALIGN_BOTTOM 5

typedef struct _ml_tagname
{
  char *name;
  int id;
} MLTagName;

typedef struct _ml_attribute
{
  char *name;
  char *value;
  struct _ml_attribute *next;
} MLAttribute;

typedef struct _ml_tag
{
  int id;                      /* tag ID */
  int end;                     /* end tag */
  char *otext;                 /* original text */
  int olen;                    /* original text length */
  char *text;                  /* processed text */
  int len;                     /* processed text length */
  MLAttribute *ta;             /* attribute list */
  struct _ml_tag *next;        /* next in parsed list */
  struct _ml_tag *qnext;       /* next in pending queue */
} MLTag;

typedef struct _ml_state
{
  WWWWidget rw;                /* */
  char *data;                  /* address of data to parse */
  int len;                     /* length of data to parse */
  char *tags;                  /* address of tag start */
  char *texts;                 /* address of tag end */
  char *cp;                    /* current position */
  int inquotes;                /* inside quotes */
  int incomment;               /* inside comment */
  MLTag *phead, *ptail;        /* parsed list */
  void (*func)();              /* called when part created */
} MLState;

#define TAG_UNKNOWN -4
#define TAG_TEXT -3
#define TAG_EOF -2
#define TAG_START -1
#define TAG_NOTHING 0
#define TAG_A 1
#define TAG_ABBREV 2
#define TAG_ABOVE 3
#define TAG_ACRONYM 4
#define TAG_ADDRESS 5
#define TAG_AROW 6
#define TAG_ARRAY 7
#define TAG_ATOP 8
#define TAG_AU 9
#define TAG_B 10
#define TAG_BASE 11
#define TAG_BELOW 12
#define TAG_BIG 13
#define TAG_BLOCKQUOTE 14
#define TAG_BODY 15
#define TAG_BOX 16
#define TAG_BQ 17
#define TAG_BR 18
#define TAG_CAPTION 19
#define TAG_CITE 20
#define TAG_CODE 21
#define TAG_DD 22
#define TAG_DFN 23
#define TAG_DIR 24
#define TAG_DL 25
#define TAG_DT 26
#define TAG_EM 27
#define TAG_FIG 28
#define TAG_FN 29
#define TAG_FORM 30
#define TAG_H1 31
#define TAG_H2 32
#define TAG_H3 33
#define TAG_H4 34
#define TAG_H5 35
#define TAG_H6 36
#define TAG_HEAD 37
#define TAG_HR 38
#define TAG_HTML 39
#define TAG_I 40
#define TAG_IMG 41
#define TAG_INPUT 42
#define TAG_ISINDEX 43
#define TAG_ITEM 44
#define TAG_KBD 45
#define TAG_LANG 46
#define TAG_LEFT 47
#define TAG_LH 48
#define TAG_LI 49
#define TAG_LINK 50
#define TAG_LISTING 51
#define TAG_MATH 52
#define TAG_MENU 53
#define TAG_META 54
#define TAG_NEXTID 55
#define TAG_NOTE 56
#define TAG_OL 57
#define TAG_OPTION 58
#define TAG_OVER 59
#define TAG_OVERLAY 60
#define TAG_P 61
#define TAG_PERSON 62
#define TAG_PLAINTEXT 63
#define TAG_PRE 64
#define TAG_Q 65
#define TAG_RIGHT 66
#define TAG_ROOT 67
#define TAG_S 68
#define TAG_SAMP 69
#define TAG_SELECT 70
#define TAG_SMALL 71
#define TAG_STRONG 72
#define TAG_STYLE 73
#define TAG_STYLES 74
#define TAG_SUB 75
#define TAG_SUP 76
#define TAG_TAB 77
#define TAG_TABLE 78
#define TAG_TD 79
#define TAG_TEXTAREA 80
#define TAG_TH 81
#define TAG_TITLE 82
#define TAG_TR 83
#define TAG_TT 84
#define TAG_U 85
#define TAG_UL 86
#define TAG_VAR 87
#define TAG_XMP 88

/*
 * Public functions
 */
MLState *MLInit _ArgProto((WWWWidget, void (*)()));
void MLAddData _ArgProto((MLState *, char *, int));
void MLDestroy _ArgProto((MLState *));
char *MLFindAttribute _ArgProto((MLTag *, char *));
int MLGetAlignID _ArgProto((char *));
void MLEndData _ArgProto((MLState *));

#endif /* __HTML_H__ */
