/*
 * io.h
 *
 * Copyright (C) 1993, 1994, 1995, John Kilburg.
 *
 * See copyright.h for more information.
 *
 */
#include "copyright.h"

#ifndef __IO_H__
#define __IO_H__ 1

typedef struct _ioinfo
{
  int s;                         /* I/O descriptor */
  void (*func)();                /* callback */
  void (*cfunc)();               /* cleanup */
  void *closure;                 /* closure for func and cfunc */

  /* line read */
  char line[BUFSIZ];             /* buffer for line IO */
  int linelen;                   /* length of line */

  /* stream read */
  void *rddata;                  /* allocated read stream data */
  int rdlen;                     /* length of read stream data */
  int rdmax;                     /* max bytes to read. */
  int rdmemsize;                 /* size of memory allocated */

  /* writing */
  void *wrdata;                  /* write data */
  int wrlen;                     /* write length */
  int wri;                       /* write index */

  WWWWidget rw;                  /* link back to widget */
  XtInputId xid;                 /* input proc ID */
  XtIntervalId iid;              /* timeout proc ID */
  struct _ioinfo *next, *prev;   /* IO list links */
} IOInfo;

typedef struct _ioqueue
{
  void (*func)();                /* callback to start IO */
  void *closure;                 /* closure for callback */
  struct _ioqueue *next;         /* queue link */
} IOQueue;

void IOStart _ArgProto((IOContext, void (*)(), void *));
void IORegister _ArgProto((IOContext, int, void (*)(), void *));
void IOStop _ArgProto((WWWWidget));
void IOEnd _ArgProto((IOContext, int));
void IOReadStream _ArgProto((IOContext, int, void (*)()));
void IOReadLine _ArgProto((IOContext, int, void (*)()));
void IOWriteStream _ArgProto((IOContext, int, void *, int, void (*)()));
void IOConnectStream _ArgProto((IOContext, int, void (*)()));
void IOResetStream _ArgProto((IOContext, int, int));
void IOSetMax _ArgProto((IOContext, int, int));

#endif /* __IO_H__ */
