/*
 * list.c
 *
 * Copyright (c) 1995, John D. Kilburg (john@cs.unlv.edu)
 *
 * See copyright.h for more information.
 *
 */
#include "copyright.h"

#include <stdio.h>

#include <X11/IntrinsicP.h>
#include <X11/StringDefs.h>

#include "WWWP.h"

/*
 *
 * Private functions
 *
 */
static void RenderLI _ArgProto((WWWWidget, RenderPart *, XEvent *));
static void SetupLI _ArgProto((WWWWidget, RenderPart *, Region, int, int));

/*
 * RenderLI
 */
static void
RenderLI(rw, rp, xe)
WWWWidget rw;
RenderPart *rp;
XEvent *xe;
{
  XFillArc(XtDisplay(rw->www.child), XtWindow(rw->www.child), rw->www.gc,
	   rp->x, rp->y,
	   rp->width, rp->height,
	   0, 360 * 64);

  return;
}

/*
 * SetupLI
 */
static void
SetupLI(rw, rp, r, line_y, line_height)
WWWWidget rw;
RenderPart *rp;
Region r;
int line_y;
int line_height;
{
  rp->y = line_y + line_height / 2;
  rp->renderable = True;
  RenderLI(rw, rp, NULL);

  return;
}

Boolean
InListItemEnv(rw, item)
WWWWidget rw;
int *item;
{
  RenderEnv *re;

  for (re = rw->www.restack; re != NULL; re = re->next)
  {
    if (re->tagid == TAG_UL || re->tagid == TAG_DL || re->tagid == TAG_OL ||
	re->tagid == TAG_MENU || re->tagid == TAG_DIR)
    {
      return(False);
    }
    else if (re->tagid == TAG_LH || re->tagid == TAG_LI ||
	     re->tagid == TAG_DT || re->tagid == TAG_DD)
    {
      *item = re->tagid;
      return(True);
    }
  }

  return(False);
}

/*
 *
 * Public functions
 *
 */
void
HandleList(rw, tagid, p, end)
WWWWidget rw;
int tagid;
MLTag *p;
int end;
{
  if (end != 0)
  {
    AddLineBreak(rw);
    DestroyRenderEnv(rw, tagid);
    AddLineBreak(rw);
  }
  else
  {
    RenderEnv *re;

    AddLineBreak(rw);
    re = CreateRenderEnv(rw, p->id);
    if (tagid != TAG_DL)
    {
      re->x += 20;
      re->width -= 20;
    }
    AddLineBreak(rw);
  }

  return;
}

void
HandleLI(rw, p, end)
WWWWidget rw;
MLTag *p;
int end;
{
  RenderPart *rp;
  XFontStruct *font;
  int item;
  RenderEnv *re;

  if (end != 0)
  {
    DestroyRenderEnv(rw, TAG_NOTHING);
    DestroyRenderEnv(rw, TAG_LI);
  }
  else
  {
    if (InListItemEnv(rw, &item)) DestroyRenderEnv2(rw, item, True);

    CreateRenderEnv(rw, TAG_LI);
    font = CurrentFont(rw);
    AddLineBreak(rw);
    
    rp = CreateRenderPart(rw, p, SetupLI, RenderLI, NULL, NULL, 0);
    rp->breakable = False;
    AddRenderPart(rw, rp, 7, 7);

    rp = CreateRenderPart(rw, NULL, NULL, NULL, NULL, NULL, 0);
    rp->breakable = False;
    AddRenderPart(rw, rp,
		  XTextWidth(font, " ", 1),
		  font->ascent + font->descent);

    re = CreateRenderEnv(rw, TAG_NOTHING);
    re->width -= (rw->www.rltail->x + rw->www.rltail->width) - re->x;
    re->x = rw->www.rltail->x + rw->www.rltail->width;
  }

  return;
}

void
HandleOL(rw, p, end)
WWWWidget rw;
MLTag *p;
int end;
{
  int item;

  if (end != 0) DestroyRenderEnv(rw, TAG_OL);
  else
  {
    if (InListItemEnv(rw, &item)) DestroyRenderEnv2(rw, item, True);
    CreateRenderEnv(rw, TAG_OL);
    AddLineBreak(rw);
  }
 
  return;
}

void
HandleDD(rw, p, end)
WWWWidget rw;
MLTag *p;
int end;
{
  int item;
  RenderEnv *re;

  if (end != 0) DestroyRenderEnv(rw, TAG_DD);
  else
  {
    if (InListItemEnv(rw, &item)) DestroyRenderEnv2(rw, item, True);
    re = CreateRenderEnv(rw, TAG_DD);
    re->x += 20;
    re->width -= 20;

    AddLineBreak(rw);
  }

  return;
}

void
HandleDT(rw, p, end)
WWWWidget rw;
MLTag *p;
int end;
{
  int item;

  if (end != 0) DestroyRenderEnv(rw, TAG_DT);
  else
  {
    if (InListItemEnv(rw, &item)) DestroyRenderEnv2(rw, item, True);
    CreateRenderEnv(rw, TAG_DT);
    AddLineBreak(rw);
  }

  return;
}

