/*
 * misc.c
 *
 * Copyright (c) 1995, John D. Kilburg (john@isri.unlv.edu)
 *
 * See copyright.h for more information.
 *
 */
#include "copyright.h"

#include <stdio.h>
#include <ctype.h>

#ifdef HAVE_STDLIB_H
#include <stdlib.h>
#endif

#include <X11/IntrinsicP.h>
#include <X11/StringDefs.h>

#include "WWWP.h"

void
HandleEOF(rw)
WWWWidget rw;
{
  AddBlankLine(rw);

  return;
}

void
HandleA(rw, p, end)
WWWWidget rw;
MLTag *p;
int end;
{
  if (end != 0) DestroyRenderEnv(rw, TAG_A);
  else
  {
    RenderEnv *re;

    if (MLFindAttribute(p, "href") != NULL)
    {
      if (InEnv(rw, TAG_A)) DestroyRenderEnv2(rw, TAG_A, True);
      
      re = CreateRenderEnv(rw, TAG_A);
      
      if (MLFindAttribute(p, "href") != NULL) re->anchorPart = p;
      if (MLFindAttribute(p, "name") != NULL) re->namePart = p;
    }
    else if (MLFindAttribute(p, "name") != NULL)
    {
      RenderPart *rp;

      rp = CreateRenderPart(rw, p, NULL, NULL, NULL, NULL, 0);
      rp->namePart = p;
      AddRenderPart(rw, rp, 0, 0);
    }
  }

  return;
}

void
HandleHx(rw, tag, p, end)
WWWWidget rw;
int tag;
MLTag *p;
int end;
{
  RenderEnv *re;

  if (end != 0)
  {
    AddBlankLine(rw);
    DestroyRenderEnv(rw, tag);
    AddBlankLine(rw);
  }
  else
  {
    char *align;

    AddBlankLine(rw);

    re = CreateRenderEnv(rw, tag);
    if (p->id == TAG_H1) re->font = rw->www.h1Font;
    else if (p->id == TAG_H2) re->font = rw->www.h2Font;
    else if (p->id == TAG_H3) re->font = rw->www.h3Font;
    else if (p->id == TAG_H4) re->font = rw->www.h4Font;
    else if (p->id == TAG_H5) re->font = rw->www.h5Font;
    else if (p->id == TAG_H6) re->font = rw->www.h6Font;

    if ((align = MLFindAttribute(p, "align")) != NULL)
    {
      re->align = MLGetAlignID(align);
    }

    AddBlankLine(rw);
  }
  
  return;
}

void
HandleFontChange(rw, end, tag)
WWWWidget rw;
int end;
int tag;
{
  RenderEnv *re;

  if (end != 0) DestroyRenderEnv(rw, tag);
  else
  {
    re = CreateRenderEnv(rw, tag);
    if (tag == TAG_EM) re->font = rw->www.iFont;
    else if (tag == TAG_B) re->font = rw->www.bFont;
    else if (tag == TAG_STRONG) re->font = rw->www.bFont;
    else if (tag == TAG_TT) re->font = rw->www.ttFont;
    else if (tag == TAG_CITE) re->font = rw->www.iFont;
    else if (tag == TAG_I) re->font = rw->www.iFont;
  }
  
  return;
}

void
HandleP(rw, p, end)
WWWWidget rw;
MLTag *p;
int end;
{
  RenderEnv *re;

  if (end != 0)
  {
    AddLineBreak(rw);
    DestroyRenderEnv(rw, TAG_P);
  }
  else
  {
    char *align;

    re = CreateRenderEnv(rw, TAG_P);
    if ((align = MLFindAttribute(p, "align")) != NULL)
    {
      re->align = MLGetAlignID(align);
    }
    AddBlankLine(rw);
    AddBlankLine(rw);
  }

  return;
}

/*
 * HandleBR
 *
 * This is function is evil.  It does things differently.
 */
void
HandleBR(rw, p, end)
WWWWidget rw;
MLTag *p;
int end;
{
  if (end == 0)
  {
    char *align;

    if ((align = MLFindAttribute(p, "align")) != NULL)
    {
      rw->www.rltail->align = MLGetAlignID(align);
    }

    AddLineBreak(rw);
  }

  return;
}

void
HandleAddress(rw, p, end)
WWWWidget rw;
MLTag *p;
int end;
{
  if (end != 0)
  {
    DestroyRenderEnv(rw, TAG_ADDRESS);
    AddLineBreak(rw);
  }
  else
  {
    RenderEnv *re;

    if (InEnv(rw, TAG_ADDRESS)) return;

    AddLineBreak(rw);
    re = CreateRenderEnv(rw, TAG_ADDRESS);
    re->font = rw->www.iFont;
  }

  return;
}


void
HandleBase(rw, p, end)
WWWWidget rw;
MLTag *p;
int end;
{
  char *url;
  URLParts *up;

  if (end == 0)
  {
    if ((url = MLFindAttribute(p, "href")) != NULL)
    {
      if ((up = ParseURL(url)) != NULL)
      {
	if (rw->www.bup != NULL) DestroyURLParts(rw->www.bup);
	rw->www.bup = up;
      }
    }
  }

  return;
}

void
HandleMisc(rw, tagid, p, end)
WWWWidget rw;
int tagid;
MLTag *p;
int end;
{
  if (end == 0) CreateRenderEnv(rw, tagid);
  else DestroyRenderEnv(rw, tagid);

  return;
}

void
HandleBlockquote(rw, p, end)
WWWWidget rw;
MLTag *p;
int end;
{
  RenderEnv *re;

  if (end != 0)
  {
    AddBlankLine(rw);
    DestroyRenderEnv(rw, TAG_BLOCKQUOTE);
    AddBlankLine(rw);
  }
  else
  {
    if (InEnv(rw, TAG_BLOCKQUOTE)) DestroyRenderEnv2(rw, TAG_BLOCKQUOTE, True);

    AddBlankLine(rw);
    re = CreateRenderEnv(rw, TAG_BLOCKQUOTE);
    re->x += re->width * 10 / 100;
    re->width -= re->width * 30 / 100;
    re->font = rw->www.iFont;
    AddBlankLine(rw);
  }

  return;
}
