/*
 * mime.c
 *
 * Copyright (C) 1994, 1995, John Kilburg (john@cs.unlv.edu)
 *
 * See copyright.h for details.
 */
#include "copyright.h"

#include <stdio.h>

#ifdef HAVE_STRING_H
#include <string.h>
#endif

#ifdef HAVE_UNISTD_H
#include <unistd.h>
#endif

#ifdef HAVE_STDLIB_H
#include <stdlib.h>
#endif

#include "common.h"
#include "wwwio.h"

typedef struct _mimetype
{
  char *content;
  char *ext;
  struct _mimetype *next;
} MIMEType;

static MIMEType def_mtlist[] =
{
  { "text/html", "html" },
  { "text/html", "htm" },
  { "text/plain", "txt" },
  { "image/gif", "gif" },
  { "image/png", "png" },
  { "image/xbm", "xbm" },
  { "image/x-xpimap", "xpm" },
  { "image/x-portable-anymap", "pnm" },
  { "image/x-portable-bitmap", "pbm" },
  { "image/x-portable-graymap", "pgm" },
  { "image/x-portable-pixmap", "ppm" },
  { "image/jpeg", "jpg" },
  { "image/jpeg", "jpeg" },
  { "image/tiff", "tiff" },
  { "image/tiff", "tif" },
  { "image/x-fits", "fit" },
  { "image/x-fits", "fits" },
  { "image/x-fits", "fts" },
  { "audio/basic", "au" },
  { "audio/basic", "snd" },
  { "text/x-compress-html", "html.Z" },
  { "text/x-gzip-html", "html.gz" },
  { "application/postscript", "ps" },
  { "application/x-dvi", "dvi" },
  { "application/x-gzip", "gz" },
  { "application/x-compress", "Z" },
  { "application/x-tar", "tar" },
  { "video/mpeg", "mpeg" },
  { "video/mpeg", "mpg" },
  { NULL, NULL },
};

static MIMEType *mtlist = NULL;

/*
 * MIMEReadTypes
 */
void
MIMEReadTypeFiles(filelist)
char *filelist;
{
  FILE *fp;
  char *f;
  char *cp;
  char buffer[256];
  char content[256];
  char exts[256];
  MIMEType *t = NULL, *c;
  char *filename, *e;

  f = filelist;
  while ((filename = mystrtok(f, ':', &f)) != NULL)
  {
    filename = FixFilename(filename);
    if (filename == NULL) continue;

    fp = fopen(filename, "r");
    if (fp == NULL) continue;
    
    while (fgets(buffer, sizeof(buffer), fp))
    {
      if (buffer[0] == '#' || buffer[0] == '\n') continue;
      
      if (sscanf(buffer, "%s %[^\n]", content, exts) == 2)
      {
	cp = exts;
	while ((e = mystrtok(cp, ' ', &cp)) != NULL)
	{
	  c = (MIMEType *)alloc_mem(sizeof(MIMEType));
	  c->content = alloc_string(content);
	  c->ext = alloc_mem(strlen(e) + 2);
	  strcpy(c->ext, ".");
	  strcat(c->ext, e);
	  c->next = NULL;
	  if (t != NULL) t->next = c;
	  else mtlist = c;
	  t = c;
	}
      }
    }
    
    fclose(fp);
  }

  return;
}

/*
 * MIMEExt2Content
 */
char *
MIMEExt2Content(ext)
char *ext;
{
  int elen, flen;
  MIMEType *c;
  int i;

  flen = strlen(ext);
  for (c = mtlist; c; c = c->next)
  {
    elen = strlen(c->ext);
    if (elen > 0)
    {
      if (elen <= flen &&
	  strncasecmp(ext + flen - elen, c->ext, elen) == 0)
      {
	return(c->content);
      }
    }
  }

  for (i = 0; def_mtlist[i].content != NULL; i++)
  {
    elen = strlen(def_mtlist[i].ext);
    if (elen > 0)
    {
      if (elen <= flen &&
	  strncasecmp(ext + flen - elen, def_mtlist[i].ext, elen) == 0)
      {
	return(def_mtlist[i].content);
      }
    }
  }

  return(NULL);
}

static MIMECap def_mclist[] =
{
  { "application/postscript", "ghostview %s" },
  { "application/x-dvi", "xdvi %s" },
  { "image/x-fits", "saoimage %s" },
  { "image/*", "xv %s" },
  { "video/mpeg", "mpeg_play -loop %s" },
  { "audio/basic", "playaudio %s" },
  { NULL, NULL },
};

static MIMECap *mclist = NULL;

/*
 * MIMEReadCapFiles
 *
 * Read a list of mailcap files.
 */
void
MIMEReadCapFiles(filelist)
char *filelist;
{
  FILE *fp;
  char *f;
  char buffer[256];
  char ctype[256];
  char command[256];
  char junk[256];
  MIMECap *c, *t = NULL;
  char *filename;

  f = filelist;
  while ((filename = mystrtok(f, ':', &f)) != NULL)
  {
    filename = FixFilename(filename);
    if (filename == NULL) continue;
   
    fp = fopen(filename, "r");
    if (fp == NULL) continue;
    
    while (fgets(buffer, sizeof(buffer), fp))
    {
      if (buffer[0] == '#' || buffer[0] == '\n') continue;
      
      if (sscanf(buffer, "%[^;]%[;]%[^;\n]", ctype, junk, command) == 3)
      {
	c = (MIMECap *)alloc_mem(sizeof(MIMECap));
	c->ctype = alloc_string(ctype);
	c->command = alloc_string(command);
	c->next = NULL;
	if (t != NULL) t->next = c;
	else mclist = c;
	t = c;
      }
    }

    fclose(fp);
  }

  return;
}

/*
 * MatchTypes
 *
 * Hell, you'd almost think that John would want to use some sort of
 * regular expression thing here instead of this shit.
 */
static int
MatchTypes(t1, t2)
char *t1, *t2;
{
  int len, dlen;

  len = strlen(t1);
  if (t1[0] != '*')
  {
    if (t1[len - 1] == '*') len--;
    if (strncasecmp(t1, t2, len) == 0) return(0);
  }
  else
  {
    dlen = strlen(t2);
    if (dlen >= len)
    {
      if (strncasecmp(t1 + 1, t2 + (dlen - len), len - 1) == 0) return(0);
    }
  }

  return(-1);
}

/*
 * FindViewer
 */
MIMECap *
MIMEFindViewer(content)
char *content;
{
  MIMECap *c;
  int i;

  for (c = mclist; c != NULL; c = c->next)
  {
    if (MatchTypes(c->ctype, content) == 0) return(c);
  }

  for (i = 0; def_mclist[i].ctype != NULL; i++)
  {
    if (MatchTypes(def_mclist[i].ctype, content) == 0) return(&def_mclist[i]);
  }

  return(NULL);
}

/*
 * StartViewer
 */
int
MIMEStartViewer(c, path)
MIMECap *c;
char *path;
{
  char *cmdline;
  int fd[2];
  int s;

  cmdline = CreateCommandLine(c->command, path, NULL);

  if (PipeCommand(cmdline, fd, 1) == 0) s = fd[0];
  else s = -1;

  free_mem(cmdline);

  return(s);
}

/*
 * ParseExpiresDate
 *
 * This functions sucks.
 */
time_t
ParseExpiresDate(s)
char *s;
{
#ifdef HAVE_MKTIME
  int i;
  time_t expires;
  struct tm tm;
  char buffer[BUFSIZ];
  char month[4], zone[4];
  static char *ms[] =
  { "Jan", "Feb", "Mar", "Apr", "May", "Jun", "Jul", "Aug", "Sep", "Oct",
    "Nov", "Dec" };

  memset(&tm, 0, sizeof(tm));
  if (sscanf(s, "%s %d %s %d %d:%d:%d %s",
	     buffer, &tm.tm_mday, month, &tm.tm_year,
	     &tm.tm_hour, &tm.tm_min, &tm.tm_sec, zone) == 8)
  {
    if (tm.tm_year > 1900) tm.tm_year -= 1900;
    for (i = 0; i < sizeof(ms) / sizeof(ms[0]); i++)
    {
      if (strcmp(month, ms[i]) == 0)
      {
	expires = mktime(&tm);
	break;
      }
    }
  }
  
  return(expires);
#else
  return(0);
#endif
}

/*
 * MIMEDestroyField
 */
void
MIMEDestroyField(m)
MIMEField *m;
{
  if (m->name != NULL) free_mem(m->name);
  if (m->value != NULL) free_mem(m->value);
  free_mem((char *)m);

  return;
}

/*
 * MIMEAddHeaderField
 */
void
MIMEAddHeaderField(d, name, value)
Document *d;
char *name;
char *value;
{
  MIMEField *m;

  m = (MIMEField *)alloc_mem(sizeof(MIMEField));
  m->name = alloc_string(name);
  m->value = alloc_string(value);
  m->used = 0;
  m->next = d->mflist;
  d->mflist = m;

  return;
}

/*
 * MIMEParseHeaderLine
 *
 * This is a disaster when it comes to parsing MIME stuff.
 */
void
MIMEParseHeaderLine(d, line)
Document *d;
char *line;
{
  char field[BUFSIZ];
  char junk[BUFSIZ];
  char data[BUFSIZ];

  if (sscanf(line, "%[^:] %[:] %[^\r\n]", field, junk, data) != 3) return;

  MIMEAddHeaderField(d, field, data);

  return;
}

/*
 * MIMEGetHeaderValue
 */
char *
MIMEGetFieldValue(mflist, name, used)
MIMEField *mflist;
char *name;
int used;
{
  MIMEField *m;

  for (m = mflist; m != NULL; m = m->next)
  {
    if (strcasecmp(m->name, name) == 0) break;
  }

  if (m != NULL)
  {
    if (used != 0)
    {
      if (m->used != 0) return(NULL);
      m->used = 1;
    }
  }
  else return(NULL);

  return(m->value);
}
